import pandas as pd
import numpy as np

from IPython.display import display, HTML, Markdown

class DataService(object):
    def __init__(self, parent, raw_data, name):
        self.parent = parent
        self.name = name
        self.raw_data = raw_data
    
    def __str__(self):
        return str(self.data)
    






import requests
import json

from transcend.user import User

def request_data(CLIENT_ID, CLIENT_SECRET, APP_ID):
    # Try not to run this too much. Requests against our server for all user data from users that authenticated your app.
    payload = {'client_id': CLIENT_ID, 'client_secret': CLIENT_SECRET, 'appId': APP_ID}
    r = requests.get('https://app.transcendbeta.com/api/group-data', params=payload, verify=False)

    if (r.status_code != 200 and r.status_code != 204):
        print "Error: " + r.text
    elif (r.status_code == 204):
        print "Request was valid, but there aren't any users on your app yet."

    splt = ',"updatedAt":'
    res = [ x for x in r.content.split(splt)]
    splt2 = '{"_id":'
    n = len(res)
    print('Number of users in the group', n - 1)

    tl = res[0][len(splt2):]
    for i in range(1, n - 1):
        spltz = res[i].split(splt2)
        good = splt2.join(spltz[:-1])
        res[i] = splt2 + tl + splt + good
        tl = spltz[-1]

    res[n - 1] = splt2 + tl + splt + res[n - 1]
    res = res[1:]

    users = [User(json.loads(x)) for x in res]
    return users


import pandas as pd
from data_service import DataService

class LinkedIn(DataService):
    def __init__(self, parent, raw_data, name='linkedin'):
        super(self.__class__, self).__init__(parent, raw_data, name)
        


'''
    How we define a user for our website
'''

from transcend.services import *

class User():
    '''
        Class to define a user
    '''
    drops = ['analytics']
    service_to_class = {
        'instagram': instagram.Instagram,
        'spotify': spotify.Spotify,
        'rescuetime': rescuetime.RescueTime,
        'moves': moves.Moves,
        'facebook': facebook.Facebook,
        'twitter': twitter.Twitter,
        'linkedin': linkedin.LinkedIn,
        'youtube': youtube.YouTube,
        'reddit': reddit.Reddit,
        'fitbit': fitbit.Fitbit,
        'underarmour': underarmour.UnderArmour
    }

    def __init__(self, user):
        '''
            Intialize user and save values
        '''
        self._id = user['_id']
        self.raw = user
        self.raw_data = {}
        for key in filter(lambda x: x not in User.drops, self.raw.keys()):
            self.raw_data[key] = self.raw[key]

        # services to be used
        self.services = map(str, self.raw_data['services'])

        # clean the data
        self.setup_services()

    def __str__(self):
        return '<Transcend-User: %s>' % self._id

    def __repr__(self):
        return '<Transcend-User: %s>' % self._id

    def setup_services(self):
        self.data = {}
        for service in self.services:
            serv_obj = data_service.DataService
            if service in User.service_to_class:
                self.data[service] = User.service_to_class[service](
                    self,
                    self.raw_data['services'][service], service)
            else:
                self.data[service] = data_service.DataService(
                    self, self.raw_data['services'][service], service)





import pandas as pd
from data_service import DataService

class Twitter(DataService):
    def __init__(self, parent, raw_data, name="twitter"):
        super(self.__class__, self).__init__(parent, raw_data, name)

    def people_you_like_most(self, MOST_COMMON=35):
        '''
            RETURNS

            [{
                'username' : 'mikeyf76' ,
                'likeCount' : 2017
            },
            ...]

        '''
        likes = self.raw_data['favorites']
        liked = map(lambda x : x['user']['screen_name'], likes)
        liked_counts = Counter(liked)
        formatted = map(lambda x: {'username': x[0], 'likeCount': x[1]}, liked_counts.most_common(MOST_COMMON))
        return formatted



import pandas as pd
import numpy as np
from data_service import DataService

from collections import Counter


class Instagram(DataService):
    '''
        Object to manipulate instagram data
    '''
    def __init__(self, parent, raw_data, name="instagram"):
        super(self.__class__, self).__init__(parent, raw_data, name)

    def posts_thru_time(self):
        '''
            Analytics for each post, typically can just create a function
            that takes in a post json object and updates the post to include 
            a new analytic

            RETURNS

            [{
                'avgComments': 2.3461538461538463,
                'avgLikes': 32.019230769230766,
                'commentsCount' : 72 ,
                'likesCount' : 2004,
                'timestamp' : datetime.datetime(2017, 1, 17, 21, 54, 19, 792000),
                'likesFollwerRatio' :
            },
            ...]

        '''
        def add_likes_follower_ratio(post, follow_count_ts):
            ts = post['timestamp']
            data_before = filter(lambda x: x[0] <= ts, follow_count_ts)

            if len(data_before) == 0:
                post['likesFollwerRatio'] = None
            else:
                followers = max(data_before, key=lambda x: x[0])[1]
                if followers > 0:
                    likes = float(post['likesCount'])
                    post['likesFollwerRatio'] = likes / followers
                else:
                    post['likesFollwerRatio'] = None
            return post

        def add_likes_comments_ra(post):
            earlier_posts = filter(lambda x: x['timestamp'] < post['timestamp'], posts)

            if len(earlier_posts) > 0:
                avg_likes = round(np.mean(map(lambda x: x['likesCount'], earlier_posts)), 3)
                avg_comments = round(np.mean(map(lambda x: x['commentsCount'], earlier_posts)), 3)
                post['avgComments'] = avg_comments
                post['avgLikes'] = avg_likes
            else:
                post['avgComments'] = None
                post['avgLikes'] = None

            return post

        posts = self.raw_data['posts']
        data = self.raw_data['data']
        follow_count_ts = map(lambda x: (x['timestamp'], x['followingCount']), data)
        formatted = map(lambda x: {
                            'commentsCount': x['commentsCount'],
                            'likesCount': x['likesCount'],
                            'timestamp': x['timestamp'],
                        },
                        posts)
        formatted = map(lambda x: add_likes_follower_ratio(x, follow_count_ts), formatted)
        formatted = map(lambda x: add_likes_comments_ra(x), formatted)

        return formatted

    def people_you_like_most(self, MOST_COMMON=35):
        '''
            RETURNS

            [{
                'username' : 'mikeyf76' ,
                'likeCount' : 2017
            },
            ...]

        '''
        likes = self.raw_data['likes']
        liked = map(lambda x: x['postedBy'], likes)
        liked_counts = Counter(liked)
        formatted = map(lambda x: {'username': x[0], 'likeCount': x[1]}, liked_counts.most_common(MOST_COMMON))
        return formatted

    def people_youre_tagged_with(self, MOST_COMMON=35):
        '''
            RETURNS

            [{
                'username' : 'mikeyf76' ,
                'tagCount' : 7
            },
            ...]

        '''
        posts = self.raw_data['posts']
        tags_per_post = map(lambda x: map(lambda y: y['username'], x['usersTagged']), posts)
        all_tags = reduce(lambda x, y: x+y, tags_per_post, [])
        top_tags = Counter(all_tags).most_common(MOST_COMMON)
        formatted = map(lambda x: {'username': x[0], 'tagCount': x[1]}, top_tags)
        return formatted

    def top_geolocations(self, MOST_COMMON=35):
        '''
            RETURNS

            [{
                'latitude' : 35.234234 ,
                'longitude' : 24.234234,
                'name' : 'Boston',
                'locCount' : 3
            },
            ...]

        '''
        posts = self.raw_data['posts']
        locations = map(lambda x: x['location'], filter(lambda x: 'location' in x, posts))
        # locations = map(lambda x : {'latitude': x['latitude'], 'longitude' : x['longitude'], 'name' : x['name']}, locations)
        key = map(lambda x: frozenset(x.items()), locations)
        top_locations = Counter(key).most_common(15)
        top_locations = map(lambda x: (dict(x[0]), x[1]), top_locations)
        formatted = map(lambda (x, y): {'latitude': x['latitude'] if 'latitude' in x else None, 
                                        'longitude': x['longitude'] if 'longitude' in x else None, 
                                        'name': x['name'] if 'name' in x else None, 
                                        'locCount': y}, top_locations)
        return formatted

    def follow_history_thru_time(self):
        '''
            RETURNS

            [{
                'followerCount' : 430 ,
                'followingCount' : 511,
                'followerFollowingRatio' : 0.841,
                'timestamp': datetime.datetime(2017, 1, 17, 21, 54, 19, 792000)
            },
            ...]

        '''
        data = self.raw_data['data']

        def get_ratio(timepoint):
            followers = timepoint['followerCount']
            following = timepoint['followingCount']
            ratio = None
            if following > 0:
                ratio = round(float(followers) / following, 3)
            timepoint['followerFollowingRatio'] = ratio
            return timepoint

        following_followers_ratio = map(get_ratio, data)
        formatted = map(lambda x: {'followerCount': x['followerCount'],
                                   'followingCount': x['followingCount'],
                                   'timestamp': x['timestamp'],
                                   'followerFollowingRatio': x['followerFollowingRatio']}, data)
        return formatted

    def top_filters(self, MOST_COMMON=35):
        '''
            RETURNS

            [{
                'filter' : 'Lo-fi' ,
                'filtCount': 17
            },
            ...]

        '''
        posts = self.raw_data['posts']
        filters = map(lambda x: x['_filter'] if '_filter' in x else None, posts)
        top_filters = Counter(filters).most_common(15)
        formatted = map(lambda (x, y): {'filter': x, 'filtCount': y}, top_filters)
        return formatted



import pandas as pd
from data_service import DataService

class Moves(DataService):
    def __init__(self, parent, raw_data, name="moves"):
        super(self.__class__, self).__init__(parent, raw_data, name)



# Compile package: python setup.py sdist
# Install package: pip install .
# Publish package: python setup.py sdist upload



from setuptools import setup
 
setup(
    name='py_transcend',   
    version='0.0.6',                       
    description= 'Python utilties for managing Transcend data.',
    author='Michael Farrell',
    author_email='mike@transcendbeta.com',
    packages = ['transcend', 'transcend/services'],
    # Choose your license
    license='MIT',
    url='https://github.com/transcend-inc/py-transcend',
)


import pandas as pd
import numpy as np
from data_service import DataService

from collections import Counter


class Spotify(DataService):
    '''
        Object to manipulate spotify data
    '''
    def __init__(self, parent, raw_data, name="spotify"):
        super(self.__class__, self).__init__(parent, raw_data, name)

    def clean_saved_tracks(self):
        def merge_two_dicts((x, y)):
            """Given two dicts, merge them into a new dict as a shallow copy."""
            y['meta'] = x
            return y
        meta_w_track = zip(self.raw_data['savedTracks']['metaData'], self.raw_data['savedTracks']['tracks'])
        saved_tracks = map(merge_two_dicts, meta_w_track)
        keys = ['energy', 'tempo', 'danceability']
        for key in keys:
            vals = map(lambda x : x['meta'][key], saved_tracks)
            if len(vals) > 0:
                mn, mx = min(vals), max(vals) 
                scaled =map(lambda x : 100. * (x - mn) / (mx - mn) if (mx - mn) > 0 else  0., vals)
                
                zp = zip(saved_tracks, scaled)
                def comb((x,y)):
                    x['meta'][key + '_scaled'] = y
                _  = map(comb, zp)

        return saved_tracks